Package: gimme
Title: Group Iterative Multiple Model Estimation
Version: 10.0
Date: 2026-04-07
Authors@R: c(person(given = "Stephanie",
                      family = "Lane",
                      role = c("aut", "trl")),
             person(given = c("Kathleen", "M"),
                      family = "Gates",
                      role = c("aut", "cre", "ccp"),
                      email = "gateskm@email.unc.edu"),
               person(given = "Zachary",
                      family = "Fisher",
                      role = "aut"),
               person(given = "Cara",
                      family = "Arizmendi",
                      role = "aut"),
               person(given = "Peter",
                      family = "Molenaar",
                      role = c("aut", "ccp")),
               person(given = "Edgar",
                      family = "Merkle",
                      role = "ctb"),
               person(given = "Michael",
                      family = "Hallquist",
                      role = "ctb"),
               person(given = "Hallie",
                      family = "Pike",
                      role = "ctb"),
               person(given = "Teague",
                      family = "Henry",
                      role = "ctb"),
               person(given = "Kelly",
                      family = "Duffy",
                      role = "ctb"),
               person(given = "Lan",
                      family = "Luo",
                      role = "ctb"),
               person(given = "Adriene",
                      family = "Beltz",
                      role = "csp"),
               person(given = "Aidan",
                      family = "Wright",
                      role = "csp"),
               person(given = "Jonathan",
                      family = "Park",
                      role = "ctb"),
               person(given = c("Sebastian", "Castro"),
                      family = "Alvarez",
                      role = "ctb"),
               person(given = c("Björn"),
                      family = "Siepe",
                      role = "ctb"))
Maintainer: Kathleen M Gates <gateskm@email.unc.edu>
Depends: R (>= 3.5.0)
Imports: lavaan (>= 0.6-17), igraph (>= 1.0-0), qgraph(>= 1.9.8),
        data.tree, MIIVsem(>= 0.5.4), imputeTS(>= 3.0), nloptr,
        graphics, stats, MASS, tseries, utils
Description: Data-driven approach for arriving at person-specific time series models. The method first identifies which relations replicate across the majority of individuals to detect signal from noise. These group-level relations are then used as a foundation for starting the search for person-specific (or individual-level) relations. See Gates & Molenaar (2012) <doi:10.1016/j.neuroimage.2012.06.026>. 
License: GPL-2
LazyData: true
URL: https://github.com/GatesLab/gimme/,
        https://tarheels.live/gimme/tutorials/
BugReports: https://github.com/GatesLab/gimme/issues
ByteCompile: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Encoding: UTF-8
Packaged: 2026-04-16 19:56:57 UTC; gateskm
Author: Stephanie Lane [aut, trl],
  Kathleen M Gates [aut, cre, ccp],
  Zachary Fisher [aut],
  Cara Arizmendi [aut],
  Peter Molenaar [aut, ccp],
  Edgar Merkle [ctb],
  Michael Hallquist [ctb],
  Hallie Pike [ctb],
  Teague Henry [ctb],
  Kelly Duffy [ctb],
  Lan Luo [ctb],
  Adriene Beltz [csp],
  Aidan Wright [csp],
  Jonathan Park [ctb],
  Sebastian Castro Alvarez [ctb],
  Björn Siepe [ctb]
Repository: CRAN
Date/Publication: 2026-04-17 14:10:02 UTC
Built: R 4.6.0; ; 2026-04-25 14:08:29 UTC; unix
