#' Small example, heterogeneous data, group and individual level effects
#'
#' This object contains a list of simulated time series data for five individuals 
#' with 50 time points and 3 variables, or regions of interest.
#'
#' @docType data
#' @keywords datasets
#' @name ts
#' @usage ts
#' @format A list of data frames with 5 individuals, who each have 50 observations on 3 variables.
NULL

#' Large example, heterogeneous data, group, subgroup, and individual level effects.
#'
#' This object contains a list of simulated time series data for twenty-five individuals 
#' with 200 time points and 10 variables, or regions of interest. 
#'
#' @docType data
#' @keywords datasets
#' @name simData
#' @usage simData
#' @format A list of data frames with 25 individuals, who each have 200 observations on 10 variables.
NULL

#' Fitted gimme object with multiple solutions
#'
#' This object contains a fitted gimme object where multiple solutions gimme was used.  The simulated
#' data had 25 individuals, each with 100 time points.
#'
#' @docType data
#' @keywords datasets
#' @name ms.fit
#' @usage ms.fit
#' @format A fitted gimme object, where multiple solutions gimme was used.
NULL

#' Latent variable example, heterogeneous data, group, subgroup level effects.
#'
#' This object contains a list of simulated time series data for twenty individuals 
#' with 500 time points and 9 variables, or regions of interest. 
#'
#' @docType data
#' @keywords datasets
#' @name simDataLV
#' @usage simDataLV
#' @format A list of data frames with 20 individuals, who each have 500 observations on 9 variables.
NULL

#' Hemodynamic Response Function (HRF) GIMME example. 
#'
#' This object contains a list of simulated time series data for twenty-five individuals.
#' Each data set has 500 time points and five variables. The fifth variable represents an onset 
#' vector for stimulation. 
#'
#' @docType data
#' @keywords datasets
#' @name HRFsim
#' @usage HRFsim
#' @format A list of data frames with 25 individuals, who each have 500 observations on 5 variables.
NULL