% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/final.org.R
\name{final.org}
\alias{final.org}
\title{Wrapup, create output files.}
\usage{
final.org(
  dat,
  grp,
  sub,
  sub_spec,
  diagnos = FALSE,
  store,
  confirm_subgroup,
  elig_paths = NULL,
  alpha = 0.05,
  indiv_correct = "Bonferroni"
)
}
\arguments{
\item{dat}{A list containing information created in setup().}

\item{grp}{A list containing group-level information. NULL in aggSEM and
indSEM.}

\item{sub}{A list containing subgroup information.}

\item{sub_spec}{A list containing information specific to each subgroup.}

\item{store}{A list containing output from indiv.search().}

\item{elig_paths}{if subgroup = TRUE, eligable paths for potential individual-level search}
}
\value{
Aggregated information, such as counts, levels, and plots.
}
\description{
Wrapup, create output files.
}
\keyword{internal}
