% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indiv.search.ms.R
\name{indiv.search.ms}
\alias{indiv.search.ms}
\title{Individual-level search. Used in gimmeSEM, aggSEM, indSEM.}
\usage{
indiv.search.ms(dat, grp, ind, ms_tol, ms_allow, grp_num)
}
\arguments{
\item{dat}{A list containing information created in setup().}

\item{grp}{A list containing group-level information. NULL in aggSEM and
indSEM.}

\item{ind}{A list containing individual- and (potentially) subgroup-level
information.}
}
\value{
Lists associated with coefficients, fit indices, etc.
}
\description{
Individual-level search. Used in gimmeSEM, aggSEM, indSEM.
}
\keyword{internal}
