% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setupBaseSyntax.R
\name{setupBaseSyntax}
\alias{setupBaseSyntax}
\title{Set up base syntax file.}
\usage{
setupBaseSyntax(paths, remove, varLabels, ctrlOpts)
}
\description{
Set up base syntax file.
}
\keyword{internal}
