% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gimmems.write.R
\name{gimmems.write}
\alias{gimmems.write}
\title{Write MS-GIMME results to data.frame.}
\usage{
gimmems.write(x)
}
\description{
Write MS-GIMME results to data.frame.
}
\keyword{internal}
