% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setupMultVars.R
\name{setupMultVarNames}
\alias{setupMultVarNames}
\title{Get names for bilinear effects.}
\usage{
setupMultVarNames(mult_vars)
}
\description{
Get names for bilinear effects.
}
\keyword{internal}
